#include <asm/io.h>
#include <stdlib.h>
#include "pcl711.h"

#define IORANGE 16

static unsigned uBaseAddr;
unsigned uGain;

/*----------------------------------------------------------------------------*/
void InitCard (int addr)
{
  /* I/O init */
  uBaseAddr = addr;

  if (ioperm (uBaseAddr,IORANGE,1))
  {
     printf ("error: I can't get access to ports 0x%X-0x%X\n",uBaseAddr,uBaseAddr+IORANGE);
     exit (EXIT_FAILURE);
  }

  Init_AD();
  Init_DA();
  Init_DI();
  Init_DO();
}
/*----------------------------------------------------------------------------*/
void CloseCard (void)
{
  ioperm (uBaseAddr,IORANGE,0);
}
/*----------------------------------------------------------------------------*/
void SetGain (unsigned gain)
{
  switch (gain)
  {
    case 1 :
      outb(0,uBaseAddr+9);
      break;
    case 2 :
      outb(1,uBaseAddr+9);
      break;
    case 4 :
      outb(2,uBaseAddr+9);
      break;
    case 8 :
      outb(3,uBaseAddr+9);
      break;
    case 16 :
      outb(4,uBaseAddr+9);
      break;
  }

  uGain = gain;
}
/*----------------------------------------------------------------------------*/
/* A/D init. */
void Init_AD(void)
{
  /* initialize analog input */
  outb(0,uBaseAddr+4);
  outb(0,uBaseAddr+5);

  /* x1 default gain */
  SetGain (1);
}
/*----------------------------------------------------------------------------*/
/* D/A init. */
void Init_DA(void)
{
  /* initialize analog output */
  outb(0,uBaseAddr+4);
  outb(0,uBaseAddr+5);
}
/*----------------------------------------------------------------------------*/
/* D/I init. */
void Init_DI(void)
{
  /* initialize digital inputs */
  outb(0,uBaseAddr+6);
  outb(0,uBaseAddr+7);
}
/*----------------------------------------------------------------------------*/
/* D/O init. */
void Init_DO(void)
{
  /* initialize digital outputs */
  outb(0,uBaseAddr+13);
  outb(0,uBaseAddr+14);
}
/*----------------------------------------------------------------------------*/
/* read A/D data with program trigger */
unsigned Read_AD(unsigned inputADNumber,float *ADValue)
{
  BYTE adhigh, adlow;
  unsigned i, timecnt;

  /* 1. set analog input inputADNumber to read in ADC */
  outb(inputADNumber,uBaseAddr+10);

  /* 
    Send to ADC read command and wait for ready state
    Bit 4(0-7) from high byte mean read end, the 12-bit read contains the
    8 low bits and 4 high bits(0-3)
  */
  /* two sequential reads to obtain a stable value */
  for (i=0; i < 2; i++)
  {
    timecnt=0;
    outb(0,uBaseAddr+12);   /* TRIGGER TWICE TO GET A STABLE DATA */
    do
    {
      adhigh=inb(uBaseAddr+5);
      if (++timecnt>=TIMEOUT) return FALSE;
    }
    while ((adhigh & 0x10)!=0);
    adlow=inb(uBaseAddr+4);
  }

  /* volts conversion */
  *ADValue=(adhigh<<8)+adlow;
  switch (uGain)
  {
    case 1 :
      *ADValue=(10*(*ADValue))/4095.0-5.0;
      break;
    case 2 :
      *ADValue=(5*(*ADValue))/4095.0-2.5;
      break;
    case 4 :
      *ADValue=(2.5*(*ADValue))/4095.0-1.25;
      break;
    case 8 :
      *ADValue=(1.25*(*ADValue))/4095.0-0.625;
      break;
    case 16 :
      *ADValue=(0.625*(*ADValue))/4095.0-0.3125;
      break;
  }

  return TRUE;
}
/*----------------------------------------------------------------------------*/
/* write analog output */
void Write_DA(unsigned DAValue)
{
  BYTE dahigh, dalow;

  dalow=(BYTE)(DAValue & (unsigned)0x00FF);
  dahigh=(BYTE)((DAValue & (unsigned)0xFF00)>>8);

  outb(dalow,uBaseAddr+4);
  outb(dahigh,uBaseAddr+5);
}
/*----------------------------------------------------------------------------*/
/* read digital data */
void Read_DI(unsigned *DIRegister)
{
  BYTE dihigh, dilow;

  dihigh=inb(uBaseAddr+7);
  dilow=inb(uBaseAddr+6);

  *DIRegister=(dihigh<<8)+dilow;
}
/*----------------------------------------------------------------------------*/
static unsigned SetBit (BYTE *byte,unsigned bit)
{
  if (bit>7) return FALSE;

  *byte=*byte | (((BYTE)0x01)<<bit);

  return TRUE;
} 
/*----------------------------------------------------------------------------*/
static unsigned ResetBit (BYTE *byte,unsigned bit)
{
  if (bit>7) return FALSE;

  switch (bit)
  {
    case 0:
      *byte=*byte & (BYTE)0xFE;
      break;
    case 1:
      *byte=*byte & (BYTE)0xFD;
      break;
    case 2:
      *byte=*byte & (BYTE)0xFB;
      break;
    case 3:
      *byte=*byte & (BYTE)0xF7;
      break;
    case 4:
      *byte=*byte & (BYTE)0xEF;
      break;
    case 5:
      *byte=*byte & (BYTE)0xDF;
      break;
    case 6:
      *byte=*byte & (BYTE)0xBF;
      break;
    case 7:
      *byte=*byte & (BYTE)0x7F;
      break;
  }
  
  return TRUE;
} 
/*----------------------------------------------------------------------------*/
/* write digital output */
unsigned Write_DO(unsigned DOBit,unsigned bValue)
{
  BYTE dohigh, dolow;
  
  if (DOBit>15) return FALSE;

  dohigh=inb(uBaseAddr+14);
  dolow=inb(uBaseAddr+13);

  if (DOBit > 7)
     if (bValue) SetBit (&dohigh,DOBit);
     else ResetBit (&dohigh,DOBit);
  else
     if (bValue) SetBit (&dolow,DOBit);
     else ResetBit (&dolow,DOBit);
  
  outb(dohigh,uBaseAddr+14);
  outb(dolow,uBaseAddr+13);

  return TRUE;
}
/*----------------------------------------------------------------------------*/
